package org.objectweb.petals.engine.performance.provider;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;

public class Receiver implements ReceiverMBean {

    protected int threadCount = 4;

    protected boolean processInContent = false;

    protected int acceptTimeout = 0;    

    /**
     * Non MBean exposed
     */
    protected ComponentContext context;
    protected DeliveryChannel channel;
    protected Logger logger;
    
    private long acceptedMessageCount = 0;
    private long notAcceptedMessageCount = 0;
    private long sentResponseCount = 0;
    private long notSentResponseCount = 0;

    protected ScheduledThreadPoolExecutor scheduledThreadPoolExecutor;

    protected BufferedWriter acceptMarker;
    
    
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.engine.performance.consumer.ReceiverMBean#startAcceptMessages()
     *
     * @throws Exception
     */
    public void startAccepting() throws Exception {
        try {
            
            // get the log directory
            File petalsroot = new File(context.getInstallRoot())
                .getParentFile().getParentFile().getParentFile()
                .getParentFile();
            File logs = new File(petalsroot, "logs");

            // init log files
            acceptMarker = new BufferedWriter(new FileWriter(new File(logs,
                "provider-accept.csv")));

            // create the tasks collection
            Collection<AcceptAndSendTask> tasks = new HashSet<AcceptAndSendTask>();
            int i = 0;
            while (i++ < threadCount) {
                tasks.add(new AcceptAndSendTask(this));
            }

            // Start the tasks in the thread worker
            scheduledThreadPoolExecutor = new ScheduledThreadPoolExecutor(
                threadCount);
            for (AcceptAndSendTask task : tasks) {
                scheduledThreadPoolExecutor.scheduleWithFixedDelay(task, 0, 1,
                    TimeUnit.MILLISECONDS);
            }
        } catch (Exception e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.engine.performance.consumer.ReceiverMBean#stopAcceptMessages()
     */
    public void stopAccepting() {
        scheduledThreadPoolExecutor.shutdown();
        try {
            scheduledThreadPoolExecutor.awaitTermination(10, TimeUnit.SECONDS);
        } catch (InterruptedException e1) {
            // do nothing
        }
        
        try {
            acceptMarker.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#reset()
     */
    public void resetCounters() {
        acceptedMessageCount = 0;
        notAcceptedMessageCount = 0;
        sentResponseCount = 0;
        notSentResponseCount = 0;
    }
    
    protected synchronized void incAcceptedMessageCount() {
        acceptedMessageCount++;
    }

    protected synchronized void incNotAcceptedMessageCount() {
        notAcceptedMessageCount++;
    }

    protected synchronized void incSentResponseCount() {
        sentResponseCount++;
    }

    protected synchronized void incNotSentResponseCount() {
        notSentResponseCount++;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#getThreadCount()
     */
    public int getThreadCount() {
        return threadCount;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#setThreadCount(int)
     */
    public void setThreadCount(int threadCount) {
        this.threadCount = threadCount;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#getTimeout()
     */
    public int getAcceptTimeout() {
        return acceptTimeout;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#setTimeout(int)
     */
    public void setAcceptTimeout(int acceptTimeout) {
        this.acceptTimeout = acceptTimeout;
    }
   
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#isProcessInContent()
     */
    public boolean isProcessInContent() {
        return processInContent;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#setProcessInContent(boolean)
     */
    public void setProcessInContent(boolean processInContent) {
        this.processInContent = processInContent;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#getAcceptedMessageCount()
     */
    public long getAcceptedMessageCount() {
        return acceptedMessageCount;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.performance.component.provider.ReceiverMBean#getNotAcceptedMessageCount()
     */
    public long getNotAcceptedMessageCount() {
        return notAcceptedMessageCount;
    }


    public long getNotSentResponseCount() {
        return notSentResponseCount;
    }

    public long getSentResponseCount() {
        return sentResponseCount;
    }
}
